// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License.

export interface ILocation {
    /**
     * Client IP address for reverse lookup
     */
    ip: string;
}