// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License.

export { AjaxMonitor as AjaxPlugin, IDependenciesPlugin, XMLHttpRequestInstrumented, IInstrumentationRequirements } from "./ajax";
export { ajaxRecord } from "./ajaxRecord";
