/*
 * Application Insights JavaScript SDK - Common, 2.6.4
 * Copyright (c) Microsoft and contributors. All rights reserved.
 */


// THIS FILE WAS AUTOGENERATED
/**
 * Identifies the source of the dependency.
 */
export var DependencySourceType;
(function (DependencySourceType) {
    DependencySourceType[DependencySourceType["Undefined"] = 0] = "Undefined";
    DependencySourceType[DependencySourceType["Aic"] = 1] = "Aic";
    DependencySourceType[DependencySourceType["Apmc"] = 2] = "Apmc";
})(DependencySourceType || (DependencySourceType = {}));
//# sourceMappingURL=DependencySourceType.js.map