// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License.

// THIS FILE WAS AUTOGENERATED
/**
 * Identifies the type of dependency.
 */
export enum DependencyKind {
    SQL = 0,
    Http = 1,
    Other = 2,
}
