"use strict";module.exports=t=>({transformIncoming:e=>e.scheme==="vscode-remote"?e.path.startsWith("/vscode-resource")?{scheme:"file",path:JSON.parse(e.query).requestResourcePath}:{scheme:"file",path:e.path}:e.scheme==="file"?{scheme:"vscode-local",path:e.path}:e,transformOutgoing:e=>e.scheme==="file"?{scheme:"vscode-remote",authority:t,path:e.path}:e.scheme==="vscode-local"?{scheme:"file",path:e.path}:e,transformOutgoingScheme:e=>e==="file"?"vscode-remote":e==="vscode-local"?"file":e});

//# sourceMappingURL=/out/serverUriTransformer.js.map
