// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License.

// THIS FILE WAS AUTOGENERATED
/**
 * Identifies the source of the dependency.
 */
export enum DependencySourceType {
    Undefined = 0,
    Aic = 1,
    Apmc = 2,
}
