/*
 * Application Insights JavaScript SDK - Common, 2.6.4
 * Copyright (c) Microsoft and contributors. All rights reserved.
 */


// THIS FILE WAS AUTOGENERATED
/**
 * Identifies the type of dependency.
 */
export var DependencyKind;
(function (DependencyKind) {
    DependencyKind[DependencyKind["SQL"] = 0] = "SQL";
    DependencyKind[DependencyKind["Http"] = 1] = "Http";
    DependencyKind[DependencyKind["Other"] = 2] = "Other";
})(DependencyKind || (DependencyKind = {}));
//# sourceMappingURL=DependencyKind.js.map